/*
 * Drizzle Client & Protocol Library
 *
 * Copyright (C) 2008 Eric Day (eday@oddments.org)
 * All rights reserved.
 *
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following disclaimer
 * in the documentation and/or other materials provided with the
 * distribution.
 * 
 *     * The names of its contributors may not be used to endorse or
 * promote products derived from this software without specific prior
 * written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * @file
 * @brief drizzle_query_st tests
 */

#include "libdrizzle-1.0/t/common.h"

#include <unistd.h>

int main(void)
{
  close(STDOUT_FILENO);

  drizzle_st *drizzle;
  drizzle_query_st *query;
  drizzle_query_st query_buffer;
  size_t size;

  printf("sizeof(drizzle_query_st) = %zu\n", sizeof(drizzle_query_st));

  if ((drizzle= drizzle_create(NULL)) == NULL)
    drizzle_test_error("drizzle_create");

  if ((query= drizzle_query_create(drizzle, &query_buffer)) == NULL)
    drizzle_test_error("drizzle_query_create");
  drizzle_query_free(query);

  if ((query= drizzle_query_create(drizzle, NULL)) == NULL)
    drizzle_test_error("drizzle_query_create");

  if (drizzle_query_options(query) != DRIZZLE_QUERY_ALLOCATED)
    drizzle_test_error("drizzle_query_options");

  drizzle_query_set_string(query, "SELECT 1+1", 10);

  if (strncmp(drizzle_query_string(query, &size), "SELECT 1+1", 10) ||
      size != 10)
  {
    drizzle_test_error("drizzle_query_string");
  }

  drizzle_query_free(query);
  drizzle_free(drizzle);

  return 0;
}
